﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Sockets;

namespace ROS.Base
{
    /// <summary>
    /// 対話クラスの基底となるもの。
    /// </summary>
    /// <remarks>
    /// <para>
    /// <see cref="ServerBase"/> クラスが結構勝手にいろいろやってしまうので、
    /// 基底としてこういうクラスがないと、継承した先でコマンド処理とかどうしたらいいかわからんので。
    /// </para>
    /// </remarks>
    public abstract class ConversationUnitBase
    {
        /// <summary>
        /// 利用する、開かれたソケット
        /// </summary>
        /// <param name="socket">利用するソケット。</param>
        /// <remarks>
        /// <para>
        /// ソケットを変えられても困るので、internalにして、他のアセンブリからは護るようにしておく。
        /// </para>
        /// </remarks>
        internal void SetSocket(Socket socket)
        {
            this.ServerSokectForClient = socket;
        }

        /// <summary>
        /// サーバクライアント間の開かれているソケット
        /// </summary>
        /// <remarks>
        /// <para>
        /// 取得しか出来ないようにしておく。
        /// </para>
        /// </remarks>
        protected Socket ServerSokectForClient { get; private set; }

        /// <summary>
        /// 対話のメソッド
        /// </summary>
        /// <remarks>
        /// <para>
        /// <see cref="ServerBase"/> はクライアントの接続を受け付けると、このメソッドをクライアント毎のスレッドで呼び出す。
        /// なので、このクラスの中に限定すれば、他のスレッドの事はあんまり考えないでいい。
        /// </para>
        /// </remarks>
        public abstract void Converse();
    }
}
