﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.Specialized;

namespace ROS.Smtp
{
    /// <summary>
    /// メールメッセージの保存のためのクラス
    /// </summary>
    /// <remarks>
    /// <para>
    /// SMTPで受信したメッセージは、解析されて、このクラスにためられる。
    /// </para>
    /// </remarks>
    [Serializable]
    public class MailMesage
    {
        /// <summary>
        /// MailMesage の新しいインスタンスを生成します。
        /// </summary>
        internal MailMesage()
        {
            this._toList = new List<string>();
            this._data = new StringBuilder();
        }

        /// <summary>
        /// 送信元アドレスをセットします。
        /// </summary>
        /// <param name="address">設定するアドレス</param>
        internal void SetFromAddress(string address)
        {
            this.FromAddress = address;
        }

        /// <summary>
        /// 送信元アドレスを取得します。
        /// </summary>
        public string FromAddress { get; private set; }

        List<string> _toList;

        /// <summary>
        /// Toのアドレスに追加します。
        /// </summary>
        /// <param name="address">追加するアドレス</param>
        /// <remarks>
        /// <para>
        /// これは複数のToがあるので、追加しかできない。
        /// 削除ないねん。めどいから。
        /// </para>
        /// </remarks>
        internal void AddToAddress(string address)
        {
            this._toList.Add(address);
        }

        /// <summary>
        /// Toのアドレスを取得します。
        /// </summary>
        public string[] ToAddresses
        {
            get
            {
                return this._toList.ToArray();
            }
        }

        StringBuilder _data;

        /// <summary>
        /// MAILのデータを追加します。
        /// </summary>
        /// <param name="data">追加するデータ</param>
        /// <remarks>
        /// <para>
        /// どんどん追加するだけのもので、別段、何かしたりはしないので注意すること。
        /// </para>
        /// </remarks>
        internal void AddData(string data)
        {
            this._data.Append(data);
        }

        /// <summary>
        /// 設定されたDataを取得します。
        /// </summary>
        public string Data
        {
            get
            {
                return this._data.ToString();
            }
        }
    }
}
