﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Net.Sockets;
using System.Collections;

namespace ROS.Smtp
{
    /// <summary>
    /// SMTP のクライアント対話ユニット。
    /// </summary>
    /// <remarks>
    /// <para>
    /// これで対話して、Mailを受信する。
    /// </para>
    /// </remarks>
    public class SmtpConversationUnit : ROS.Base.ConversationUnitBase
    {
        //override
        #region Converse / 対話するところの本体
        /// <summary>
        /// 対話するところの本体
        /// </summary>
        /// <remarks>
        /// <para>
        /// ここ、try-catchしないでも、呼び出し側でやってくれるので、がーっと書いちゃえ。
        /// </para>
        /// </remarks>
        public override void Converse()
        {
            //処理を開始するで
            this.SendMessageToClient(this.ServerSokectForClient, SmtpMessages.OK220);

            SmtpCommandData command = new SmtpCommandData();

            List<MailMesage> mailMessageList = new List<MailMesage>();
            MailMesage mailMessage = null;

            while (true)
            {
                bool isFollowConverse = false;
                byte[] byteArray = new byte[2048];


                   if (!this.ServerSokectForClient.Connected)
                   {
                       break;
                   }

                //要するにデータが正常にきてないんだ
                //.のあと、QUITしてくれない



                    int receivedLength = this.ServerSokectForClient.Receive(byteArray, byteArray.Length, 0);

                string messageFromClient = Encoding.ASCII.GetString(byteArray, 0, receivedLength);

                if (receivedLength > 0)
                {
                    //コマンドをパーズする
                    command.ParseCommandFromMessage(messageFromClient);
                }
                else
                {
                    //終わっていいべさ
                    command.SetQuitCommand();
                }

                if (command.LastCommand == SmtpCommands.RequestAccepted && command.NewCommand == SmtpCommands.Rset)
                {
                    //データ受信完了してるのにRESTが来たら、前のやつはとっとこう
                    if (mailMessage != null)
                    {
                        mailMessageList.Add(mailMessage);
                        mailMessage = new MailMesage();
                    }
                }
                else
                {
                    if (command.NewCommand == SmtpCommands.Mail && mailMessage == null)
                    {
                        //Mailが来たら、新規で作成するなり
                        mailMessage = new MailMesage();
                    }

                    //パーズしたコマンドの処理してー
                    isFollowConverse = SmtpHandleCommands.HandleCommand(command.NewCommand, messageFromClient, this.ServerSokectForClient, mailMessage);
                }

                if (!isFollowConverse)
                {
                    //もう終わっていいさ
                    break;
                }
            }

            //データたまっていたら追加しておく
            if (mailMessage != null)
            {
                mailMessageList.Add(mailMessage);
            }

            //おわったよー
            OnReceiveCompleted(mailMessageList.ToArray());
        } 
        #endregion

        //受信完了のイベント周り
        #region OnReceiveCompleted と、イベント周り
        /// <summary>
        /// メールの受信を通知する <see cref="ReceiveCompleted"/> のためのイベントハンドラ。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベントデータ</param>
        public delegate void ReceiveCompletedHandler(object sender, ReceiveCompletedEventArgs e);

        /// <summary>
        /// メール受信完了時に呼び出されるイベントです。このイベントは、スレッドセーフに呼び出されます。
        /// </summary>
        public event ReceiveCompletedHandler ReceiveCompleted;

        /// <summary>
        /// メール受信完了時にイベントを呼び出す際に、複数のスレッドが呼び出し元をいっぺんに呼び出さないようにするためのもの
        /// </summary>
        private static object receiveCompletedLockObject = new object();
        
        /// <summary>
        /// 受信が完了した時に呼び出され、<see cref="ReceiveCompleted"/> イベントを発生させます。
        /// </summary>
        /// <param name="mailMessages">受信したメールメッセージ</param>
        protected virtual void OnReceiveCompleted(MailMesage[] mailMessages)
        {
            if (ReceiveCompleted != null)
            {
                //イベントは分割して起こす事にする
                foreach (MailMesage mail in mailMessages)
                {
                    lock (receiveCompletedLockObject)
                    {
                        ReceiveCompleted(this, new ReceiveCompletedEventArgs(mail));
                    }
                }
            }
        }
        #endregion

        //ヘルパ
        #region SendMessageToClient / クライアントにメッセージを送信します。
        /// <summary>
        /// クライアントにメッセージを送信します。
        /// </summary>
        /// <param name="sokectToUse">利用するソケット</param>
        /// <param name="message">送信するメッセージ</param>
        protected virtual void SendMessageToClient(Socket sokectToUse, string message)
        {
            byte[] byteArray = Encoding.ASCII.GetBytes(message.ToCharArray());
            sokectToUse.Send(byteArray, byteArray.Length, 0);
        } 
        #endregion

        //ヘルパというにはでかい、class
        //この子はクライアントからのメッセージをパーズして、SMTPコマンドに切り分けたりしてくれる子
        #region protected class SmtpCommandData
        /// <summary>
        /// クライアントからのメッセージをParseして、SMTPコマンドを引っこ抜くひと
        /// </summary>
        protected class SmtpCommandData
        {
            #region constructor
            /// <summary>
            /// SmtpCommandData の新しいインスタンスを生成します。
            /// </summary>
            public SmtpCommandData()
            {
                this.NewCommand = null;
                this.LastCommand = null;
            }
            #endregion

            /// <summary>
            /// 新しいSMTPコマンド
            /// </summary>
            public string NewCommand { get; protected set; }

            /// <summary>
            /// 前回のSMTPコマンド
            /// </summary>
            public string LastCommand { get; protected set; }


            #region ParseCommandFromMessage / メッセージから、コマンドを分析する
            /// <summary>
            /// メッセージから、コマンドを分析する
            /// </summary>
            /// <param name="messageFromClient">受け取ったクライアントメッセージ</param>
            public void ParseCommandFromMessage(string messageFromClient)
            {
                //終わりになってれば終わる
                if (IsDataEnd(messageFromClient))
                {
                    this.LastCommand = SmtpCommands.DataEnd;
                    this.NewCommand = SmtpCommands.RequestAccepted;        //おわったぞな
                }
                else if (messageFromClient.Trim().Length == 0)
                {
                    //メッセージない時は終わりで
                    this.NewCommand = SmtpCommands.Quit;
                }
                else if (this.LastCommand == SmtpCommands.Data || this.LastCommand == SmtpCommands.DataFollow)
                {
                    //別のコマンドで処理されるまで、内部的にデータが流れている最中にしておく
                    this.NewCommand = SmtpCommands.DataFollow;
                }
                else
                {
                    //SMTPコマンドくれよ
                    this.NewCommand = messageFromClient.Substring(0, 4).Trim();
                }

                //正しいコマンドの処理であるかチェックしておく > ここで、LastCommandを設定するで
                ChangeValidSMTPCommandSequence();
            }

            #region ChangeValidSMTPCommandSequence / SMTPコマンドが順序よく流れてきているかのチェックして、書き換えちゃう
            /// <summary>
            /// SMTPコマンドが順序よく流れてきているかのチェックして、書き換えちゃう
            /// </summary>
            /// <remarks>
            /// <para>
            /// これ、LastCommandを設定するお仕事がメイン
            /// </para>
            /// <para>
            /// 書き換えちゃうと書いてあるように、順序が正しくないと書き換えてQUITとかにしてしまうので注意
            /// </para>
            /// </remarks>
            private void ChangeValidSMTPCommandSequence()
            {
                if (this.LastCommand == null)
                {
                    if (this.NewCommand != SmtpCommands.HELO &&
                        this.NewCommand != SmtpCommands.EHLO &&
                        this.NewCommand != SmtpCommands.Noop &&
                        this.NewCommand != SmtpCommands.Vrfy)
                    {
                        //おかしい話なのでー
                        this.NewCommand = SmtpCommands.Quit;
                    }
                    else
                    {
                        //いれておく
                        this.LastCommand = this.NewCommand;
                    }
                }
                else
                {
                    //なんかもっときれいにかけないものかなぁ > できるんだろうが、めどいので放置する
                    switch (this.NewCommand)
                    {
                        case SmtpCommands.Mail:
                            if (this.LastCommand == SmtpCommands.HELO ||
                                this.LastCommand == SmtpCommands.EHLO ||
                                this.LastCommand == SmtpCommands.Noop ||
                                this.LastCommand == SmtpCommands.Vrfy)
                            {
                                this.LastCommand = this.NewCommand;
                            }
                            break;
                        case SmtpCommands.Rcpt:
                            if (this.LastCommand == SmtpCommands.Mail ||
                                this.LastCommand == SmtpCommands.Noop ||
                                this.LastCommand == SmtpCommands.Vrfy)
                            {
                                this.LastCommand = this.NewCommand;
                            }
                            break;
                        case SmtpCommands.Data:
                            if (this.LastCommand == SmtpCommands.Rcpt ||
                                this.LastCommand == SmtpCommands.Noop ||
                                this.LastCommand == SmtpCommands.Vrfy)
                            {
                                this.LastCommand = this.NewCommand;
                            }
                            break;
                        case SmtpCommands.DataFollow:
                            if (this.LastCommand == SmtpCommands.Data ||
                                this.LastCommand == SmtpCommands.DataFollow)
                            {
                                this.LastCommand = this.NewCommand;
                            }
                            break;
                        case SmtpCommands.RequestAccepted:
                            if (this.LastCommand == SmtpCommands.DataFollow)
                            {
                                this.LastCommand = this.NewCommand;
                            }
                            break;
                    }
                }
            }
            #endregion

            #endregion

            //ヘルパ的なもの
            #region IsDataEnd / データが終わりかどうか
            /// <summary>
            /// データが終わりかどうか
            /// </summary>
            /// <param name="messageFromClient">クライアントからのメッセージ</param>
            /// <returns>終わりのデータであれば <c>true</c> 。</returns>
            /// <remarks>
            /// <para>
            /// 手抜きで、VisualBasic名前空間を使っているんで、ここだけ直せば、VisualBasic名前空間はいらない。
            /// </para>
            /// </remarks>
            private bool IsDataEnd(string messageFromClient)
            {
                return (Microsoft.VisualBasic.Strings.InStrRev(messageFromClient,
                    SmtpCommands.CrLf + SmtpCommands.DataEnd, -1, Microsoft.VisualBasic.CompareMethod.Binary) > 0);
            }
            #endregion

            #region SetQuitCommand / SMTP QUITのコマンドをセットする
            /// <summary>
            /// SMTP QUITのコマンドをセットする
            /// </summary>
            public void SetQuitCommand()
            {
                this.NewCommand = SmtpCommands.Quit;
            }
            #endregion
        } 
        #endregion
    }
}
