﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Net.Sockets;
using System.IO;

namespace ROS.Smtp
{
    /// <summary>
    /// SMTPコマンドを処理して、クライアントに返すクラス。
    /// </summary>
    /// <remarks>
    /// <para>
    /// これはコマンドをうにうに処理するためだけのクラス。
    /// 全部static。
    /// </para>
    /// </remarks>
    public class SmtpHandleCommands
    {
        //関数ポインタ的に使うので
        private delegate bool HandleCommandDelegate(string clientMessage, Socket serverSocketForClient, MailMesage mailMessage);

        /// <summary>
        /// SmtpHandleCommands のメソッドが利用される時に呼び出されるコンストラクタ的なもの。
        /// </summary>
        /// <remarks>
        /// <para>
        /// 呼び出しの定義をするだけ。
        /// </para>
        /// </remarks>
        static SmtpHandleCommands()
        {
            CommandTable = new Hashtable();

            CommandTable.Add(SmtpCommands.HELO, new HandleCommandDelegate(CommandHELLO));
            CommandTable.Add(SmtpCommands.EHLO, new HandleCommandDelegate(CommandEHLO));
            CommandTable.Add(SmtpCommands.Mail, new HandleCommandDelegate(CommandMAIL));
            CommandTable.Add(SmtpCommands.Rcpt, new HandleCommandDelegate(CommandRCPT));
            CommandTable.Add(SmtpCommands.Data, new HandleCommandDelegate(CommandDATA));
            CommandTable.Add(SmtpCommands.DataFollow, new HandleCommandDelegate(CommandDATAFLLOW));
            CommandTable.Add(SmtpCommands.RequestAccepted, new HandleCommandDelegate(CommandREQUESTACCEPTED));
            CommandTable.Add(SmtpCommands.Noop, new HandleCommandDelegate(CommandNOOP));
            CommandTable.Add(SmtpCommands.Vrfy, new HandleCommandDelegate(CommandVRFY));
            CommandTable.Add(SmtpCommands.Rset, new HandleCommandDelegate(CommandRSET));
            CommandTable.Add(SmtpCommands.Quit, new HandleCommandDelegate(CommandQUIT));
        }

        //これのsetはいらんのだがなー
        private static Hashtable CommandTable { get; set; }

        /// <summary>
        /// コマンドを解析し、適切なメッセージをSocketを通じて、クライアントに送信します。
        /// 必要であれば、メッセージの内容をStreamに追加します。
        /// </summary>
        /// <param name="command">コマンド</param>
        /// <param name="clientMessage">受信メッセージ</param>
        /// <param name="serverSocketForClient">利用しているソケット。</param>
        /// <param name="mailMessage">書き込まれるストリーム</param>
        /// <returns>処理を続ける必要があれば、<c>true</c> 。</returns>
        public static bool HandleCommand(string command, string clientMessage, Socket serverSocketForClient, MailMesage mailMessage)
        {
            if (CommandTable.Contains(command))
            {
                return ((HandleCommandDelegate)CommandTable[command])(clientMessage, serverSocketForClient, mailMessage);
            }

            return false;
        }


        //こっから下は、コマンドに対しての応答とか

        private static bool CommandHELLO(string clientMessage, Socket serverSocketForClient, MailMesage mailMessage)
        {
            SendMessageToClient(serverSocketForClient, SmtpMessages.OK250);
            return true;
        }

        private static bool CommandEHLO(string clientMessage, Socket serverSocketForClient, MailMesage mailMessage)
        {
            SendMessageToClient(serverSocketForClient, SmtpMessages.OK250);
            return true;
        }

        private static bool CommandMAIL(string clientMessage, Socket serverSocketForClient, MailMesage mailMessage)
        {
            if (mailMessage == null)
            {
                //これは変
                throw new ArgumentNullException("mailMessage");
            }

            mailMessage.SetFromAddress(clientMessage.Trim());
            SendMessageToClient(serverSocketForClient, SmtpMessages.OK250);

            return true;
        }

        private static bool CommandRCPT(string clientMessage, Socket serverSocketForClient, MailMesage mailMessage)
        {
            if (mailMessage == null)
            {
                //これは変
                throw new ArgumentNullException("mailMessage");
            }

            mailMessage.AddToAddress(clientMessage.Trim());

            SendMessageToClient(serverSocketForClient, SmtpMessages.OK250);
            return true;
        }

        private static bool CommandDATA(string clientMessage, Socket serverSocketForClient, MailMesage mailMessage)
        {
            SendMessageToClient(serverSocketForClient, SmtpMessages.StartMailInput354);
            return true;
        }

        private static bool CommandDATAFLLOW(string clientMessage, Socket serverSocketForClient, MailMesage mailMessage)
        {
            if (mailMessage == null)
            {
                //これは変
                throw new ArgumentNullException("mailMessage");
            }

            mailMessage.AddData(clientMessage);
            return true;
        }

        private static bool CommandREQUESTACCEPTED(string clientMessage, Socket serverSocketForClient, MailMesage mailMessage)
        {
            if (mailMessage == null)
            {
                //これは変
                throw new ArgumentNullException("mailMessage");
            }

            mailMessage.AddData(clientMessage);
            SendMessageToClient(serverSocketForClient, SmtpMessages.OK250);

            //終わる

            return true;
        }

        private static bool CommandNOOP(string clientMessage, Socket serverSocketForClient, MailMesage mailMessage)
        {
            SendMessageToClient(serverSocketForClient, SmtpMessages.OK250);
            return true;
        }

        private static bool CommandVRFY(string clientMessage, Socket serverSocketForClient, MailMesage mailMessage)
        {
            SendMessageToClient(serverSocketForClient, SmtpMessages.AddressUnknown550);
            return true;
        }

        private static bool CommandRSET(string clientMessage, Socket serverSocketForClient, MailMesage mailMessage)
        {
            SendMessageToClient(serverSocketForClient, SmtpMessages.OK250);
            return true;
        }

        private static bool CommandQUIT(string clientMessage, Socket serverSocketForClient, MailMesage mailMessage)
        {
            SendMessageToClient(serverSocketForClient, SmtpMessages.ClosingConnection221);
            return false;
        }
   
        /// <summary>
        /// クライアントにメッセージを送信します。
        /// </summary>
        /// <param name="sokectToUse">利用するソケット</param>
        /// <param name="message">送信するメッセージ</param>
        private static void SendMessageToClient(Socket sokectToUse, string message)
        {
            byte[] byteArray = Encoding.ASCII.GetBytes(message.ToCharArray());
            sokectToUse.Send(byteArray, byteArray.Length, 0);
        }
    }
}
