﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.Sockets;

namespace ROS.Smtp
{
    /// <summary>
    /// SMTPサーバの実装
    /// </summary>
    /// <remarks>
    /// 基本的にはこのクラスを使ってほにゃららするか、このクラスを継承して、ほにゃららする。
    /// </remarks>
    public class SmtpServer : ROS.Base.ServerBase
    {
        #region constructor
        /// <summary>
        /// SmtpServer の新しいインスタンスを生成します。
        /// </summary>
        /// <param name="waitAddress">待機するアドレス</param>
        /// <param name="waitPort">待機するポート</param>
        public SmtpServer(IPAddress waitAddress, int waitPort)
        {
            this._serverState = new ROS.Base.ServerState(waitAddress, waitPort);
        } 
        #endregion

        //prop
        #region State / サーバの状態を取得します。

        private readonly ROS.Base.ServerState _serverState;

        /// <summary>
        /// サーバの状態を取得します。
        /// </summary>
        protected override ROS.Base.IServerSettings State
        {
            get { return this._serverState; }
        } 
        #endregion

        #region ServerType / サーバーのタイプを返す
        /// <summary>
        /// サーバーのタイプを返す
        /// </summary>
        public override string ServerType
        {
            get { return "SMTP"; }
        } 
        #endregion

        //override
        #region IsAcceptClientThreadStart / 問い合わされた接続の接続を許可するか
        /// <summary>
        /// 問い合わされた接続の接続を許可するか
        /// </summary>
        /// <param name="serverSokectForClient">クライアントとの接続</param>
        /// <returns>接続を許可するならtrue。</returns>
        /// <remarks>
        /// <para>
        /// IPベースで制限するとか、このあたりでやるとよい。
        /// </para>
        /// </remarks>
        protected override bool IsAcceptClientThreadStart(Socket serverSokectForClient)
        {
            return base.IsAcceptClientThreadStart(serverSokectForClient);
        } 
        #endregion

        #region CreateConversationUnit / Smtp対話用のユニットを生成して返す
        /// <summary>
        /// Smtp対話用のユニットを生成して返す
        /// </summary>
        /// <returns>生成されたSmtp用の対話ユニット</returns>
        /// <remarks>
        /// <para>
        /// ベースクラスはこのユニットを使って、メッセージをほにゃららする。
        /// </para>
        /// </remarks>
        protected override ROS.Base.ConversationUnitBase CreateConversationUnit()
        {
            SmtpConversationUnit unit = new SmtpConversationUnit();

            if (this.ReceiveCompleted != null)
            {
                unit.ReceiveCompleted += ReceiveCompleted;
            }

            return unit;
        }
        #endregion

        //event
        #region ReceiveCompleted / 受信終了時に呼び出されるイベント。このイベントはスレッドセーフです。
        /// <summary>
        /// 受信終了時に呼び出されるイベント。このイベントはスレッドセーフです。
        /// </summary>
        public event SmtpConversationUnit.ReceiveCompletedHandler ReceiveCompleted;
        #endregion
    }
}
